
from atlas_engine_client.core.api import Client

from .keywords import DeployKeyword
from .keywords import EmptyTaskKeyword
from .keywords import EngineKeyword
from .keywords import ExternalTaskKeyword
from .keywords import ManualTaskKeyword
from .keywords import ProcessInstanceKeyword
from .keywords import SendKeyword
from .keywords import StartKeyword
from .keywords import UserTaskKeyword

from robot.api import logger

from .docker_handler import DockerHandler


class ProcessCubeLibrary(DeployKeyword, EmptyTaskKeyword, 
    EngineKeyword, ExternalTaskKeyword, ManualTaskKeyword, 
    ProcessInstanceKeyword, SendKeyword, StartKeyword, 
    UserTaskKeyword):

    ROBOT_LIBRARY_SCOPE = 'GLOBAL'

    def __init__(self, **kwargs):
        self._engine_url = kwargs.get('engine_url', None)
        self._self_hosted_engine = kwargs.get('self_hosted_engine', None)
        self._docker_options = kwargs.get('docker_options', {})

        self._max_retries = int(kwargs.get('max_retries', 5))
        self._backoff_factor = int(kwargs.get('backoff_factor', 2))
        self._delay = float(kwargs.get('delay', 0.1))
        self._worker_id = kwargs.get('worker_id', "robot_framework")

        self._client = self._create_client(**kwargs)

        super(ProcessCubeLibrary, self).__init__(self._client, **kwargs)

    def _create_client(self, **kwargs):
        engine_url = kwargs.get('engine_url', None)
        self_hosted_engine = kwargs.get('self_hosted_engine', None)

        client = None

        if engine_url:
            logger.console(f"Connecting to engine '{engine_url}'.")
            client = Client(engine_url)
        elif self_hosted_engine == 'docker':
            docker_options = kwargs.get('docker_options', {})

            logger.console(f"Starting engine within a docker container ...")
            self._docker_handler = DockerHandler(**docker_options)
            self._docker_handler.start()
            logger.console(
                f"Starting engine within a docker container ... done")

            engine_url = self._docker_handler.get_engine_url()

            logger.console(f"Connecting to engine '{engine_url}'.")
            client = Client(engine_url)
        else:
            raise TypeError(
                "No 'engine_url' or 'self_hosted_engine' parameter provided.")

        return client

    def engine_shutdown(self):
        if self._docker_handler:
            self._docker_handler.shutown()



import time

from atlas_engine_client.core.api import ManualTaskQuery

from robot.api import logger

from ._retry_helper import retry_on_exception
class ManualTaskKeyword:

    def __init__(self, client, **kwargs):
        self._client = client

        self._max_retries = kwargs.get('max_retries', 5)
        self._backoff_factor = kwargs.get('backoff_factor', 2)
        self._delay = kwargs.get('delay', 0.1)

    @retry_on_exception
    def get_manual_task_by(self, **kwargs):

        logger.debug(kwargs)

        query = ManualTaskQuery(**kwargs)

        logger.info(query)

        current_retry = 0
        current_delay = self._delay

        while True:
            manual_tasks = self._client.manual_task_get(query)

            logger.info(manual_tasks)

            if len(manual_tasks) >= 1:
                manual_task = manual_tasks[0]
            else:
                manual_task = {}

            if manual_task:
                break
            else:
                time.sleep(current_delay)
                current_retry = current_retry + 1
                current_delay = current_delay * self._backoff_factor
                if current_retry > self._max_retries:
                    break
                logger.info(
                    f"Retry count: {current_retry}; delay: {current_delay}")

        return manual_task

    @retry_on_exception
    def finish_manual_task(self, manual_task_instance_id: str):
        self._client.manual_task_finish(manual_task_instance_id)



#!/usr/bin/env python

import setuptools

try:
    from pip.req import parse_requirements
except ImportError:  # pip >= 10.0.0
    from pip._internal.req import parse_requirements

parsed_reqs = parse_requirements('requirements.txt', session='hack')
installed_reqs = [str(ir.requirement) for ir in parsed_reqs]

with open("README.md", "r") as fh:
    long_description = fh.read()

name = "5Minds IT-Solutions GmbH & Co. KG"

setuptools.setup(
    name="robotframework-processcube",
    version=setuptools.sic('1.3.0-alpha.5'),
    description="Robot Framework Keywords for processcube workflow engine.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=name,
    author_email="processcube@5minds.de",
    url="https://github.com/atlas-engine-contrib/robotframework-processcube",
    packages=setuptools.find_packages(),
    install_requires=installed_reqs,
    classifiers=[
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Framework :: Robot Framework",
    ],
    license="MIT Licence",
)
